@@ -165,16 +165,48 @@ check_if_syslinux_runs() {
   $sys_exec -h 2>/dev/null >/dev/null
   lrc=$?
   if [ "$lrc" -ne 0 ]; then
-   if [ "$(LC_ALL=C uname -m)" = "x86_64" ]; then
-     [ "$BOOTUP" = "color" ] && $SETCOLOR_FAILURE
-     echo "On x86-64 system, you should install libc6-i386 (for Debian/Ubuntu) or glibc.i686 (for Fedora/CentOS/OpenSuSE) package so that the required libraries to run 32-bit program $sys_exec exist."
-     [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
-     echo "Program terminated!"
-     exit 1
-   fi
+    [ "$BOOTUP" = "color" ] && $SETCOLOR_FAILURE
+    echo "File to run $sys_exec."
+    [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
+    echo "Program terminated!"
+    exit 1
   fi
   return $lrc
 } # end of check_if_syslinux_runs
+#
+is_partition() {
+  # //NOTE// If new RE* is added, remember to check the function check_if_disk_busy_before_create_partition and get_not_busy_disks_or_parts have to be modified, too.
+  # IDE and SCSI disk partition: /dev/hda1, /dev/hdb1, /dev/sda1, /dev/sdb1...
+  # KVM virtual disk partition: /dev/vda1, /dev/vdb1...
+  # Xen virtual disk partition: /dev/xvda1, /dev/xvdb1, /dev/xdvc1...
+  # CCISS RAID disk partition: /dev/cciss/c0d0p1, /dev/cciss/c0d1p1...
+  # SD card: /dev/mmcblk0p1, /dev/mmcblk0p2, /dev/mmcblk0p3...
+  # NBD device: /dev/nbd0p1, /dev/nbd0p2...
+  # NVME device: /dev/nvme0n1p1, /dev/nvme0n1p2, /dev/nvme1n1p1, /dev/nvme1n1p2
+  # FakeRAID: with nodmraid boot parameter, /dev/md126p1, /dev/md126p2...
+  # Mylex ExtremeRAID-2000 SCSI RAID controller: /dev/rd/c0d0p1, /dev/rd/c0d1p1...
+  # Compaq Smart Array controller: /dev/ida/c0d0p1, /dev/ida/c0d1p2...
+  local dev_=${1#/dev/*}
+  local RE1='x?[hsv]d[a-z]+[0-9]+'
+  local RE2='i2o/hd[a-z]+[0-9]+'
+  local RE3='cciss/c[0-9]d[0-9]p[0-9]+'
+  local RE4='mmcblk[0-9]+p+[0-9]+'
+  local RE5='md[0-9]+p+[0-9]+'
+  local RE6='rd/c[0-9]d[0-9]p[0-9]+'
+  local RE7='ida/c[0-9]d[0-9]p[0-9]+'
+  local RE8='nvme[0-9]+n[0-9]+p+[0-9]+'
+  local RE9='nbd[0-9]+p+[0-9]+'
+  local rc=1
+  local fs_
+
+  if [ -z "$dev_" ]; then
+    return 1
+  fi
+  echo $dev_ | grep -Eq "^($RE1|$RE2|$RE3|$RE4|$RE5|$RE6|$RE7|$RE8|$RE9)$"
+  rc=$?
+
+  return $rc
+} # end of is_partition
 
 #
 export LANG=C
@@ -211,6 +243,10 @@ done
 
 #
 check_if_root
+if [ "$#" -ne 1 ]; then
+  USAGE
+  exit 1
+fi
 target_part="$1"
 
 if ! type parted &>/dev/null; then
@@ -227,6 +263,13 @@ if ! type blkid &>/dev/null; then
   echo "Program terminated!"
   exit 1
 fi
+if ! type mcopy &>/dev/null; then
+  [ "$BOOTUP" = "color" ] && $SETCOLOR_FAILURE
+  echo 'Program "mcopy" was not found on this GNU/Linux system. Please install it. On Debian or Ubuntu Linux, you can install it by: sudo apt-get install mtools'
+  [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
+  echo "Program terminated!"
+  exit 1
+fi
 
 if [ -n "$target_part_label" -a -z "$target_part" ] ; then
   target_part_via_label="$(LC_ALL=C blkid | grep -iE "^/dev/[hsu][bd][a-z]+[[:digit:]]+.+LABEL=\"$target_part_label\".*" | grep -oE -w '^/dev/[hsu][bd][a-z]+[[:digit:]]+'| head -n 1)"
@@ -251,7 +294,7 @@ if [ -z "$target_part" ]; then
   exit 1
 fi
 # Make sure target_part is partition device name, not disk device name
-if [ -z "$(echo $target_part | grep -iE "/dev/[hsu][bd][a-z]+[[:digit:]]+")" ]; then
+if ! is_partition $target_part ; then
   [ "$BOOTUP" = "color" ] && $SETCOLOR_FAILURE
   echo "\"$target_part\" is NOT a valid partition name!"
   [ "$BOOTUP" = "color" ] && $SETCOLOR_NORMAL
@@ -259,6 +302,12 @@ if [ -z "$(echo $target_part | grep -iE "/dev/[hsu][bd][a-z]+[[:digit:]]+")" ];
   exit 1
 fi
 
+# Get the running OS arch
+os_arch="$(LC_ALL=C uname -m)"
+case "$os_arch" in
+	x86_64)    arch_d="x64" ;;
+	i[3456]86) arch_d="x86" ;;
+esac
 #
 pt_dev="$(basename $target_part)"  # e.g. sdc1
 hd_dev="$(get_diskname $target_part)"   # e.g. sdc
@@ -427,7 +476,7 @@ linux_tmpd="$(mktemp -d /tmp/linux_tmp.XXXXXX)"
 [ $? -eq 0 ] || exit 1
 
 echo "A filesystem supporting Unix file mode for $mode is required. Copying $mode to $linux_tmpd"
-cp -fv "$path_of_prog/utils/linux/$mode" "$linux_tmpd"
+cp -fv "$path_of_prog/utils/linux/$arch_d/$mode" "$linux_tmpd"
 chmod u+x "$linux_tmpd/$mode"
 
 check_if_syslinux_runs "$linux_tmpd/$mode"
